lg = love.graphics
fs = love.filesystem
function love.load()
	fs.setIdentity("BPM Tapper")

	bpm = 0
	count = 0
	fullCount = 16
	done = false
	bar = 3
	timeStart = 0
	lastTime = 0
	reset = 2
	seg = nil 
	setup()

	flash = 0

	config = {
		sound = true
	}

	if fs.getInfo("config.lua") then
		config = fs.load("config.lua")()
	else
		saveConfig()
	end

	color = {
		fg = {0.2, 0.2, 0.2},
		bg = {0.1, 0.1, 0.1},
		txt = {0.8, 0.8, 0.8},
		ac = {0.3, 0.7, 0.9},
		shadow = {0, 0, 0, 0.2}
	}

	lg.setBackgroundColor(color.bg)

	beep = love.audio.newSource("click.ogg", "static")
end

function setup()
	font = lg.newFont("VCR_OSD_MONO.ttf", lg.getWidth() * 0.15)
	font:setFilter("nearest", "nearest")
	fontHeight = font:getAscent() - font:getDescent()
	shadowDistance = fontHeight * 0.12
	lg.setFont(font)
	lg.setLineWidth(fontHeight * 0.1)

	pump = lg.getWidth() * 0.05
end

function tap()
	local time = love.timer.getTime()
	if (time - lastTime > reset) then
		count = 0
		bpm = 0
		bar = 3
	end

	flash = 1
	if count == 0 then
		timeStart = time
		count = count + 1
	else
		bpm = 60 * count / (time - timeStart)
		count = count + 1
	end
	lastTime = time

	--Sound
	if config.sound then
		bar = bar + 1
		if bar > 4 then bar = 1 end

		if bar == 4 then
			beep:setPitch(1.3)
		else
			beep:setPitch(1)
		end

		beep:stop()
		beep:play()
	end
end

function sprint(t)
	s = s..t.."\n"
end

function love.update(dt)
	flash = flash + (0 - flash) * (5 * dt)
end

function love.draw()
	local cx = lg.getWidth() / 2
	local cy = lg.getHeight() / 2
	local bpmString = math.floor(bpm * 100) / 100

	local rad = (lg.getWidth() / 3) + (pump * flash)

	lg.setColor(color.shadow)
	lg.circle("fill", cx, cy + shadowDistance, rad, seg)

	lg.setColor(color.fg)
	lg.circle("fill", cx, cy, rad, seg)

	lg.setColor(color.ac)
	lg.circle("line", cx, cy, rad, seg)

	local r = (rad / fullCount) * count
	if r > rad then
		r = rad
	end
	lg.setColor(color.ac)
	lg.circle("fill", cx, cy, r, seg)

	lg.setColor(1, 1, 1, flash)
	lg.circle("fill", cx, cy, rad * (1 - flash), seg)

	lg.setColor(color.shadow)
	lg.printf(bpmString, 0, cy - (fontHeight / 2) + shadowDistance, lg.getWidth(), "center")

	lg.setColor(color.txt)
	lg.printf(bpmString, 0, cy - (fontHeight / 2), lg.getWidth(), "center")

	local str = "SOUND ON"
	if not config.sound then
		str = "SOUND OFF"
	end
	lg.setColor(color.ac)
	lg.printf(str, 0, lg.getHeight() * 0.9, lg.getWidth(), "center")

	lg.setColor(1, 1, 1, 0.1)
	lg.rectangle("fill", 0, lg.getHeight() * 0.9, lg.getWidth(), lg.getHeight() * 0.1)
end

function love.mousepressed(x, y, key)
	if y > lg.getHeight() * 0.9 then
		config.sound = not config.sound
		saveConfig()
	else
		tap()
	end
end

function love.resize(w, h)
	setup()
end

function love.keypressed(key)
	if key == "escape" then
		love.event.push("quit")
	end
	tap()
end

function saveConfig()
	fs.write("config.lua", tableToString(config))
end

function tableLength(tab)
	local length = 0
	for k,v in pairs(tab) do
		length = length + 1
	end
	return length
end

function tableToString(tab, recursion)
	recursion = recursion or false
	local t = "return {"
	local comma = false
	local length = tableLength(tab)
	local i = 0
	if recursion then t = "{" end

	for k, v in pairs(tab) do
		i = i + 1
		local val = tostring(v)
		if type(v) == "string" then
			val = '"'..v..'"'
		elseif type(v) == "table" then
			val = tableToString(v, true)
		end

		t = t..k.." = "..val
		if i < length then
			t = t..", "
		end
	end

	t = t.."}"

	return t
end